#include <DHT.h>
#include <LiquidCrystal_I2C.h>
#include <WiFi.h>
#include <WiFiClient.h>
#include <WebServer.h>
#include <ESPmDNS.h>

#include "pitches.h"
#include "html.h"

// DHT sensor 

#define DHTPIN 32
#define DHTTYPE DHT22

DHT dht(DHTPIN, DHTTYPE);

// LCD board

#define LCD_ADDRESS 0x27
#define LCD_COLUMNS 16
#define LCD_ROWS 2

LiquidCrystal_I2C lcd(LCD_ADDRESS, LCD_COLUMNS, LCD_ROWS);  

// ESP32 web server

#define SSID "costin_andrei.vlad"
#define PASSWORD "pmfair2024"

WebServer server(80);

// flame and gas sensors

#define LDRPIN 35
#define MQ2PIN 34
#define FLAME_THRESHOLD 10
#define GAS_THRESHOLD 50

// passive buzzer

#define BUZZER_PIN 18
#define NO_NOTES 3
#define NOTE_C6 1047

// buttons  

#define RESET_BUTTON_PIN 19
#define DEBOUNCE_TIME 1000

int lastResetTime = 0;

// global fire variables

bool fire = false;

// ----------------------------------

void IRAM_ATTR handle_reset()
{
  if (fire) {
    int time = millis();
    
    if (time - lastResetTime >= DEBOUNCE_TIME) {
      Serial.println("Alarm Reset");

      fire = false;

      lastResetTime = time;
    }
  }
}

void handleRoot() {
  char msg[1500];

  if (fire) {
    snprintf(msg, 1500, fire_message);
  } else {
    snprintf(msg, 1500, normal_message, dht.readTemperature(), dht.readHumidity());
  }

  server.send(200, "text/html", msg);
}

void setup(){
  Serial.begin(115200);

  // DHT Sensor init
  dht.begin();

  // LCD init
  lcd.init();
  lcd.backlight();
  
  // Web Server init
  WiFi.mode(WIFI_STA);
  WiFi.begin(SSID, PASSWORD);
  Serial.println("");

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.print("Connected to ");
  Serial.println(SSID);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  if (MDNS.begin("esp32")) {
    Serial.println("MDNS responder started");
  }

  server.on("/", handleRoot);

  server.begin();
  Serial.println("HTTP server started");

  // Button init
  pinMode(RESET_BUTTON_PIN, INPUT_PULLUP);
  attachInterrupt(RESET_BUTTON_PIN, handle_reset, FALLING);
}

void update_lcd() {
  lcd.clear();

  if (fire) {
    lcd.setCursor(0, 0);
    lcd.print("   !! FIRE !!   ");

    lcd.setCursor(0, 1);
    lcd.print(" !! GET  OUT !! ");
  } else {
    float temp = dht.readTemperature();
    float humid = dht.readHumidity();

    lcd.setCursor(0, 0);
    lcd.print("Temp: ");
    lcd.print(temp, 1);
    lcd.print(" ");
    lcd.print((char)223);
    lcd.print("C");

    lcd.setCursor(0, 1);
    lcd.print("Humidity: ");
    lcd.print(humid, 1);
    lcd.print(" %");
  }
}

void update_alarm() {
  if (fire) {
    for (int thisNote = 0; thisNote < NO_NOTES; thisNote++) {
      tone(BUZZER_PIN, NOTE_C6, 1000);

      int pauseBetweenNotes = 1000 * 1.2;

      delay(pauseBetweenNotes);

      noTone(BUZZER_PIN);
    }
  }
}

void check_fire() {
  int valf = analogRead(LDRPIN);
  valf = map(valf, 0, 4095, 0, 100);
  Serial.println("Value flame: ");
  Serial.println(valf);

  int valg = analogRead(MQ2PIN);
  valg = map(valg, 0, 4095, 0, 100);
  Serial.println("Value gas: ");
  Serial.println(valg);

  if (valf < FLAME_THRESHOLD || valg > GAS_THRESHOLD) {
    fire = true;
  }
}

void loop(){
  update_lcd();

  update_alarm();

  server.handleClient();

  check_fire();
  
  delay(1000);
}